﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Web;

namespace APITestPages.Components
{
  public class Utils
  {
    private HttpWebRequest webRequest;
    private HttpWebResponse webResponse;

    public static string HMAC(string pSecret, string pMerchantId, string pTerminalId, string pSeed, string pAmount, string pTimeStamp, string pVersion)
    {
      if (pTimeStamp == "")
        return HMACSHA512(pSecret, pMerchantId + ":" + pTerminalId + ":" + pSeed + ":" + pAmount);
      else
        return HMACSHA512(pSecret, pMerchantId + ":" + pTerminalId + ":" + pSeed + ":" + pAmount + ":" + pTimeStamp);
    }


    public static string HMACSHA512(string key, string textToHash)
    {
      if (string.IsNullOrEmpty(key))
        throw new ArgumentNullException("HMACSHA512: key", "Parameter cannot be empty.");
      if (string.IsNullOrEmpty(textToHash))
        throw new ArgumentNullException("HMACSHA512: textToHash", "Parameter cannot be empty.");
      if (key.Length % 2 != 0 || key.Trim().Length < 2)
      {
        throw new ArgumentNullException("HMACSHA512: key", "Parameter cannot be odd or less than 2 characters.");
      }
      try
      {
        // This is the section to con vert byte array to hexadecimal string
        byte[] k = Enumerable.Range(0, key.Length)
                    .Where(x => x % 2 == 0)
                    .Select(x => Convert.ToByte(key.Substring(x, 2), 16))
                    .ToArray();
        HMACSHA512 hmac = new HMACSHA512(k);
        byte[] HashedValue = hmac.ComputeHash((new System.Text.ASCIIEncoding()).GetBytes(textToHash));
        return BitConverter.ToString(HashedValue).Replace("-", string.Empty);
      }
      catch (Exception ex)
      {
        throw new Exception("HMACSHA512: " + ex.Message);
      }
    }

    public String submitRequest(string URL, String request)
    {
      String hostResponse = null;
      StreamReader streamReader = null;
      webRequest = null;
      webResponse = null;

      try
      {
        System.Net.ServicePointManager.Expect100Continue = false;
        //webRequest = (HttpWebRequest)HttpWebRequest.Create(requestURL + "?" + request);
        webRequest = (HttpWebRequest)HttpWebRequest.Create(URL);
        webRequest.Method = "POST";
        webRequest.Headers.Clear();
        //webRequest.UserAgent = "Mozilla/4.0 (compatible; Win32; WinHttp.WinHttpRequest.5)";
        webRequest.UserAgent = "ChargeAnywhere";
        webRequest.Accept = "*/*";
        webRequest.Headers.Add(HttpRequestHeader.KeepAlive, "300");
        webRequest.KeepAlive = true;
        webRequest.ContentType = "application/x-www-form-urlencoded; Charset=UTF-8";
        webRequest.ContentLength = request.Length;
        //webRequest.Timeout = 30000;
        StreamWriter writer = new StreamWriter(webRequest.GetRequestStream());
        writer.Write(request);
        writer.Flush();
        writer.Close();

        webResponse = (HttpWebResponse)webRequest.GetResponse();

        streamReader = new StreamReader(webResponse.GetResponseStream());
        hostResponse = streamReader.ReadToEnd();
      }
      catch (System.Threading.ThreadAbortException tab) { }
      catch (Exception ex)
      {

        //Response.Write(ex.Message);
        return "";
      }

      if (hostResponse != null)
        streamReader.Close();
      if (webResponse != null)
        webResponse.Close();

      return hostResponse;
    }

    public static String SanitizeAmount(String pAmount)
    {
      decimal value = 0;
      try
      {
        value = Decimal.Parse(pAmount, NumberStyles.Currency);
      }
      catch
      {
      }
      return value.ToString("F2");
    }

    public static double ToDouble(String pAmount)
    {
      decimal value = Decimal.Parse(pAmount, NumberStyles.Currency);
      return Convert.ToDouble(value);
    }

    public static String FormatCurrency(String pAmount)
    {
      return ToDouble(SanitizeAmount(pAmount)).ToString("C");
    }
    public static String FormatCurrency(double pAmount)
    {
      return pAmount.ToString("C");
    }
  }
}